library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following lines to use the declarations that are
-- provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity checksum_generator is
  port (
    clk   : in  std_logic;
    -- Synchronous clear
    clr   : in  std_logic;
    -- Asynchronous reset
    rst   : in  std_logic;
    -- Input data, 8 bits wide
    data  : in  std_logic_vector(7 downto 0);
    -- Enable: start accumulating and adding
    strb  : in  std_logic;
    -- 8-bit output, updated at every clock.
    cksum : out std_logic_vector(7 downto 0)
    );
end checksum_generator;

architecture Behavioral of checksum_generator is
  signal sum : std_logic_vector(8 downto 0);
begin
  process (clk, rst)
  begin
    if (rst = '1') then
      sum   <= "000000000";
    elsif (rising_edge(clk)) then
      if (clr = '1') then
        sum <= "000000000";
      elsif (strb = '1') then
        -- The accumulated value is just the sum of all bytes so far
        -- received, with a carry.  Note that the carry stage is
        -- technically off-by-one, meaning that we must feed this
        -- module an additional 0x00 at the end, by using the end_checksum
        -- input.  Otherwise we would need some carry lookahead logic.
        sum <= ('0'&(sum(7 downto 0))) + data + sum(8);
      end if;
    end if;
  end process;

  -- purpose: Output the checksum, based on the current value of the
  --          accumulator
  -- type   : combinational
  -- inputs : acc
  -- outputs: cksum
  output_cksum : process (sum)
  begin  -- process output_cksum
    cksum <= not sum(7 downto 0);
  end process output_cksum;

end Behavioral;
