library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

-- This module is used for simulating a LIN network
entity lin_network is
  port (
    addr1   : in    std_logic_vector (3 downto 0);
    addr2   : in    std_logic_vector (3 downto 0);
    clk     : in    std_logic;
    r_n1    : in    std_logic;
    r_n2    : in    std_logic;
    rst     : in    std_logic;
    sel_n1  : in    std_logic;
    sel_n2  : in    std_logic;
    w_n1    : in    std_logic;
    w_n2    : in    std_logic;
    data1   : inout std_logic_vector (7 downto 0);
    data2   : inout std_logic_vector (7 downto 0);
    lin_bus : inout std_logic;
    int1    : out   std_logic;
    int2    : out   std_logic
    );
end lin_network;

architecture structural of lin_network is
  component lin_controller
    Port (
      clk   : in    std_logic;
      rst   : in    std_logic;
      sel_n : in    std_logic;
      r_n   : in    std_logic;
      w_n   : in    std_logic;
      int   : out   std_logic;
      addr  : in    std_logic_vector (3 downto 0);
      data  : inout std_logic_vector (7 downto 0);
      RxD   : in    std_logic;
      TxD   : out   std_logic
    );
  end component;
  
  component lin_transceiver
    port (
      TxD     : in    std_logic;
      RxD     : out   std_logic;
      lin_bus : inout std_logic
    );
  end component;

  signal RxD1, TxD1 : std_logic;
  signal RxD2, TxD2 : std_logic;
begin
  lin_controller1: lin_controller port map (
    clk, rst, sel_n1, r_n1, w_n1, int1, addr1, data1, RxD1, TxD1
  );
  lin_controller2: lin_controller port map (
    clk, rst, sel_n2, r_n2, w_n2, int2, addr2, data2, RxD2, TxD2
  );
  lin_transceiver1 : lin_transceiver port map (
    TxD1, RxD1, lin_bus
  );
  lin_transceiver2 : lin_transceiver port map (
    TxD2, RxD2, lin_bus
  );

end structural;
