library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following lines to use the declarations that are
-- provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity majority_sampler is
  port (
    clk  : in  std_logic;
    -- Asynchronous reset
    rst  : in  std_logic;
    -- Input
    data : in  std_logic;
    -- Output
    dout : out std_logic
  );
end majority_sampler;

-- Meant for use with a 16 x oversampling rate. The output will go
-- high when 9 of the last 16 samples have gone high. 
architecture Behavioral of majority_sampler is
  signal num : std_logic_vector (3 downto 0);
begin
  process (clk, rst)
  begin
    if (rst = '1') then
      num     <= "1111";
    elsif (rising_edge(clk)) then
      if (data = '1' and num /= "1111") then
        num <= num + 1;
      elsif (data = '0' and num /= "0000") then
        num <= num - 1;
      else
        num <= num;
      end if;
    end if;
  end process;

  dout <= '1' when (num >= "1000") else '0';
  
end Behavioral;
