library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following lines to use the declarations that are
-- provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity receiver is
  port (
    bitclk_x16    : in  std_logic;
    rst           : in  std_logic;
    ser_in        : in  std_logic;
    done          : out std_logic;
    -- parallel output
    par_out       : out std_logic_vector (7 downto 0);
    framing_error : out std_logic
    );
end receiver;

architecture Behavioral of receiver is
  -- 9-bit wide shift register
  signal shift_register : std_logic_vector(8 downto 0);
  signal counter        : std_logic_vector (3 downto 0);
  signal done_int       : std_logic;
  signal bitclk         : std_logic;
  signal divide_counter : std_logic_vector (2 downto 0);
-- the receiver module operates at 16x the bit clock.
begin
  -- internally divide the 16x bitclk to bitclk
  divider : process (bitclk_x16, rst)
  begin
    if (rst = '1') then
      divide_counter <= "000";
      bitclk <= '0';
    elsif (rising_edge(bitclk_x16)) then
      divide_counter <= divide_counter + 1;
      if (divide_counter = "111") then
        bitclk <= not (bitclk) after 10 ns;
      end if;
    end if;
  end process divider;
  
  receiver : process (bitclk, rst)
  begin
    if (rst = '1') then
      shift_register <= "000000000";
      counter <= "0000";
    elsif (rising_edge(bitclk)) then
      if (done_int = '0') then
        shift_register <= ser_in & shift_register (8 downto 1);
        counter <= counter + 1;
      end if;
    end if;
    
  end process receiver;

  -- start bit, 8 bits, stop bit = 10 bits
  done_int <= '1' when (counter = "1010") else '0';
  -- To Do: check here for a lack of stop bit
  done <= done_int;
  par_out <= shift_register (7 downto 0);
  framing_error <= '1' when (shift_register(8) = '0' and done_int = '1') else
                   '0';

end Behavioral;
