library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following lines to use the declarations that are
-- provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity transmitter is
  port (
    bitclk       : in  std_logic;
    rst          : in  std_logic;
    bypass_input : in  std_logic;
    ser_in       : in  std_logic;
    ser_out      : out std_logic;
    done         : out std_logic;
    bypass       : in  std_logic;
    -- parallel input
    par_in       : in  std_logic_vector (7 downto 0);
    par_load     : in  std_logic;
    xmit_error   : out std_logic
    );
end transmitter;

architecture Behavioral of transmitter is
  -- 10-bit wide shift register
  signal shift_register        : std_logic_vector(8 downto 0);
  signal counter               : std_logic_vector (3 downto 0);
  signal done_reg, ser_out_reg : std_logic;
-- the transmitter module operates at 1x the bit clock.
begin
  -- Check ser_in to see if the bit was transmitted correctly.  A
  -- failure here could indicate a bus failure, like a shorted bus or
  -- a faulty tranceiver.
  xmit_check : process (bitclk, rst)
  begin
    if (rst = '1') then
      xmit_error <= '0';
    elsif (rising_edge(bitclk)) then
      if (par_load = '1') then
        xmit_error <= '0';
      elsif (done_reg = '0' and ser_out_reg /= ser_in) then
          xmit_error <= '1';
      end if; 
--      if (done_reg = '0') then
--        if (par_load = '1') then
--          xmit_error <= '0';
--        elsif (ser_out_reg /= ser_in) then
--          xmit_error <= '1';
--        end if;
    end if;
  end process xmit_check;

  transmitter : process (bitclk, rst)
  begin
    if (rst = '1') then
      shift_register <= "111111111";
      counter <= "1010";
    elsif (rising_edge(bitclk)) then
      if (par_load = '1') then
        -- load the shift register - start bit is '0'
        shift_register <= par_in & '0';
        counter <= "0000";
      elsif (done_reg = '0') then
        -- fill the shift register with 1's - shortcut for stop bit
        shift_register <= '1' & shift_register (8 downto 1);
        counter <= counter + 1;
      else
        counter <= counter;
        shift_register <= shift_register;
      end if;
    end if;
  end process transmitter;

  -- output logic
  done_reg <= '1' when (counter = "1010") else '0';
  done <= done_reg;
  -- serial output is the low bit of the shift register
  ser_out_reg <= bypass_input when (bypass='1') else
                 '1' when (done_reg = '1') else
                 shift_register (0);
  ser_out <= ser_out_reg;

end Behavioral;
